@@ -1,6 +1,6 @@
 (* camlp5r *)
-(* $Id: eprinter.ml 1392 2007-10-13 00:31:18Z deraugla $ *)
-(* Copyright (c) INRIA 2007 *)
+(* $Id: eprinter.ml 1822 2007-12-28 12:58:30Z deraugla $ *)
+(* Copyright (c) INRIA 2007-2008 *)
 
 type t 'a =
   { pr_name : string;
@@ -10,7 +10,10 @@ and pr_level 'a = { pr_label : string; pr_rules : mutable pr_rule 'a }
 and pr_rule 'a =
   Extfun.t 'a (pr_fun 'a -> pr_fun 'a -> pr_context -> string)
 and pr_fun 'a = pr_context -> 'a -> string
-and pr_context = { ind : int; bef : string; aft : string; dang : string };
+and pr_context =
+  Pprintf.pr_context ==
+    { ind : int; bef : string; aft : string; dang : string }
+;
 
 type position =
   [ First
@@ -99,8 +102,6 @@ value clear pr = do {
 value apply_level pr lname pc z = pr.pr_fun lname pc z;
 value apply pr pc z = pr.pr_fun "" pc z;
 
-value empty_pc = {ind = 0; bef = ""; aft = ""; dang = ""};
-
 value print pr =
   List.iter
     (fun lev -> do {
