@@ -1,6 +1,6 @@
 (* camlp5r *)
-(* $Id: fstream.ml 1558 2007-11-23 03:55:32Z deraugla $ *)
-(* Copyright 2007 INRIA *)
+(* $Id: fstream.ml 1830 2007-12-29 03:52:53Z deraugla $ *)
+(* Copyright (c) INRIA 2007-2008 *)
 
 type t 'a = { count : int; data : Lazy.t (data 'a) }
 and data 'a =
@@ -119,4 +119,4 @@ value b_term f strm =
   | None -> None ]
 ;
 
-value b_nok = K (fun _ -> None);
+value b_act a strm = Some (a, strm, K (fun _ -> None));
