@@ -1,6 +1,6 @@
 (* camlp5r pa_lexer.cmo *)
-(* $Id: plexer.ml 1571 2007-11-26 18:23:11Z deraugla $ *)
-(* Copyright (c) INRIA 2007 *)
+(* $Id: plexer.ml 1804 2007-12-27 10:30:24Z deraugla $ *)
+(* Copyright (c) INRIA 2007-2008 *)
 
 value no_quotations = ref False;
 value error_on_unknown_keywords = ref False;
@@ -106,16 +106,11 @@ value number =
   | decimal_digits_under end_integer! ]
 ;
 
-value rec char_aux ctx bp =
-  lexer
-  [ "'"/
-  | _ (char_aux ctx bp)!
-  | -> err ctx (bp, $pos) "char not terminated" ]
-;
+value char_aux = lexer [ "'"/ | _ [ "'"/ | _ [ "'"/ | ] ] ];
 
 value char ctx bp =
   lexer
-  [ "\\" _ (char_aux ctx bp)!
+  [ "\\" _ char_aux!
   | "\\" -> err ctx (bp, $pos) "char not terminated"
   | ?= [ _ '''] _! "'"/ ]
 ;
