@@ -1,12 +1,13 @@
 (* camlp5r pa_extend.cmo q_MLast.cmo *)
-(* $Id: pa_fstream.ml 1558 2007-11-23 03:55:32Z deraugla $ *)
+(* $Id: pa_fstream.ml 1812 2007-12-27 19:50:50Z deraugla $ *)
 
 open Pcaml;
 
 type spat_comp =
   [ SpTrm of MLast.loc and MLast.patt and option MLast.expr
   | SpNtr of MLast.loc and MLast.patt and MLast.expr
-  | SpStr of MLast.loc and MLast.patt ]
+  | SpStr of MLast.loc and MLast.patt
+  | SpWhn of MLast.loc and MLast.expr ]
 ;
 type sexp_comp =
   [ SeTrm of MLast.loc and MLast.expr
@@ -51,7 +52,9 @@ value stream_pattern_component skont =
                 [ $p$ -> $skont$
                 | _ -> None ] >>
   | SpStr loc p ->
-      <:expr< let $p$ = $lid:strm_n$ in $skont$ >> ]
+      <:expr< let $p$ = $lid:strm_n$ in $skont$ >>
+  | SpWhn loc e ->
+      <:expr< if $e$ then $skont$ else None >> ]
 ;
 
 value rec stream_pattern loc epo e =
@@ -119,77 +122,118 @@ value rec cstream loc =
       <:expr< Fstream.flazy $slazy loc x$ >> ]
 ;
 
-(* backtracking parsers *)
+(* meta parsers *)
 
 value patt_expr_of_patt p =
   let loc = MLast.loc_of_patt p in
   match p with
   [ <:patt< $lid:x$ >> -> (p, <:expr< $lid:x$ >>)
   | <:patt< $uid:_$ $lid:x$ >> -> (<:patt< $lid:x$ >>, <:expr< $lid:x$ >>)
-  | _ -> (<:patt< () >>, <:expr< () >>) ]
+  | <:patt< ($_$ as $lid:x$) >> -> (<:patt< $lid:x$ >>, <:expr< $lid:x$ >>)
+  | _ -> (<:patt< _ >>, <:expr< () >>) ]
 ;
 
-value bstream_pattern_component skont =
+value no_compute =
   fun
-  [ SpTrm loc p wo ->
-      let (p1, e1) = patt_expr_of_patt p in
-      let e =
-        <:expr< Fstream.b_term (fun [ $p$ -> Some $e1$ | _ -> None ]) >>
-      in
-      <:expr< Fstream.b_seq $e$ (fun $p1$ -> $skont$) >>
-  | SpNtr loc p e ->
-      <:expr< Fstream.b_seq $e$ (fun $p$ -> $skont$) >>
-  | SpStr loc p ->
-      Ploc.raise loc (Stream.Error "not impl: stream_pattern_component") ]
+  [ <:expr< $lid:_$ >> -> True
+  | <:expr< $uid:_$.$lid:_$ >> -> True
+  | _ -> False ]
 ;
 
-value rec bstream_pattern loc (spcl, epo, e) =
+value fun_p_strm_e loc p e =
+  match (p, e) with
+  [ (<:patt< $lid:a$ >>, <:expr< $e$ $lid:b$ $lid:strm_n$ >>)
+    when a = b && no_compute e -> e
+  | (p, <:expr< $e$ $lid:strm_n$ >>)
+    when no_compute e ->
+      <:expr< fun $p$ -> $e$ >>
+  | _ ->
+      <:expr< fun $p$ $lid:strm_n$ -> $e$ >> ]
+;
+
+value fun_strm_e_strm loc e =
+  if no_compute e then e else <:expr< fun $lid:strm_n$ -> $e$ $lid:strm_n$ >>
+;
+
+value mstream_pattern_component m skont =
+  fun
+  [ SpTrm loc p1 wo ->
+      let (p, e) = patt_expr_of_patt p1 in
+      let e =
+        match p1 with
+        [ <:patt< _ >> ->
+            (* prevent compiler warning *)
+            <:expr< $uid:m$.b_term (fun _ -> Some $e$) >>
+        | _ ->
+            <:expr< $uid:m$.b_term (fun [ $p1$ -> Some $e$ | _ -> None ]) >> ]
+      in
+      let e2 = fun_p_strm_e loc p skont in
+      <:expr< $uid:m$.b_seq $e$ $e2$ $lid:strm_n$ >>
+  | SpNtr loc p e ->
+      let e2 = fun_p_strm_e loc p skont in
+      <:expr< $uid:m$.b_seq $fun_strm_e_strm loc e$ $e2$ $lid:strm_n$ >>
+  | SpStr loc p ->
+      Ploc.raise loc (Stream.Error "not impl: stream_pattern_component 1")
+  | SpWhn loc e ->
+      <:expr< if $e$ then $skont$ else None >> ]
+;
+
+value rec mstream_pattern loc m (spcl, epo, e) =
   match spcl with
   [ [] ->
-      let e =
-        let ek = <:expr< Fstream.b_nok >> in
-        let e = <:expr< Some ($e$, $lid:strm_n$, $ek$) >> in
-        match epo with
-        [ Some p -> <:expr< let $p$ = Fstream.count $lid:strm_n$ in $e$ >>
-        | None -> e ]
-      in
-      <:expr< fun $lid:strm_n$ -> $e$ >>
+      let e = <:expr< $uid:m$.b_act $e$ $lid:strm_n$ >> in
+      match epo with
+      [ Some p -> <:expr< let $p$ = $uid:m$.count $lid:strm_n$ in $e$ >>
+      | None -> e ]
   | [spc :: spcl] ->
-      let skont = bstream_pattern loc (spcl, epo, e) in
-      bstream_pattern_component skont spc ]
+      let skont = mstream_pattern loc m (spcl, epo, e) in
+      mstream_pattern_component m skont spc ]
 ;
 
-value bparser_cases loc spel =
-  let rel = List.rev_map (bstream_pattern loc) spel in
+value mparser_cases loc m spel =
+  let rel = List.rev_map (mstream_pattern loc m) spel in
   match rel with
-  [ [e :: rel] ->
+  [ [e] -> e
+  | [e :: rel] ->
       let e =
-        List.fold_left (fun e e1 -> <:expr< Fstream.b_or $e1$ $e$ >>) e rel
+        List.fold_left
+          (fun e e1 -> <:expr< $uid:m$.b_or (fun $lid:strm_n$ -> $e1$) $e$ >>)
+          <:expr< fun $lid:strm_n$ -> $e$ >> rel
       in
       <:expr< $e$ $lid:strm_n$ >>
   | [] ->
       <:expr< None >> ]
 ;
 
-value bparser_match loc me bpo pc =
-  let pc = bparser_cases loc pc in
-  let e =
-    match bpo with
-    [ Some bp -> <:expr< let $bp$ = Fstream.count $lid:strm_n$ in $pc$ >>
-    | None -> pc ]
-  in
-  <:expr< let ($lid:strm_n$ : Fstream.t _) = $me$ in $e$ >>
+value rec is_not_bound s =
+  fun
+  [ <:expr< $uid:_$ >> -> True
+  | _ -> False ]
 ;
 
-value bparser loc bpo pc =
-  let e = bparser_cases loc pc in
+value mparser_match loc m me bpo pc =
+  let pc = mparser_cases loc m pc in
   let e =
     match bpo with
-    [ Some bp -> <:expr< let $bp$ = Fstream.count $lid:strm_n$ in $e$ >>
+    [ Some bp -> <:expr< let $bp$ = $uid:m$.count $lid:strm_n$ in $pc$ >>
+    | None -> pc ]
+  in
+  let p =
+    if is_not_bound strm_n e then <:patt< _ >>
+    else <:patt< $lid:strm_n$ >>
+  in
+  <:expr< let ($p$ : $uid:m$.t _) = $me$ in $e$ >>
+;
+
+value mparser loc m bpo pc =
+  let e = mparser_cases loc m pc in
+  let e =
+    match bpo with
+    [ Some bp -> <:expr< let $bp$ = $uid:m$.count $lid:strm_n$ in $e$ >>
     | None -> e ]
   in
-  let p = <:patt< ($lid:strm_n$ : Fstream.t _) >> in
-  <:expr< fun $p$ -> $e$ >>
+  let p = <:patt< ($lid:strm_n$ : $uid:m$.t _) >> in
+   <:expr< fun $p$ -> $e$ >>
 ;
 
 EXTEND
@@ -208,15 +252,15 @@ EXTEND
           <:expr< $cparser_match loc e po [pc]$ >>
       | "bparser"; po = OPT ipatt; "["; pcl = LIST0 parser_case SEP "|";
         "]" ->
-          <:expr< $bparser loc po pcl$ >>
+          <:expr< $mparser loc "Fstream" po pcl$ >>
       | "bparser"; po = OPT ipatt; pc = parser_case ->
-          <:expr< $bparser loc po [pc]$ >>
+          <:expr< $mparser loc "Fstream" po [pc]$ >>
       | "match"; e = SELF; "with"; "bparser"; po = OPT ipatt; "[";
         pcl = LIST0 parser_case SEP "|"; "]" ->
-          <:expr< $bparser_match loc e po pcl$ >>
+          <:expr< $mparser_match loc "Fstream" e po pcl$ >>
       | "match"; e = SELF; "with"; "bparser"; po = OPT ipatt;
         pc = parser_case ->
-          <:expr< $bparser_match loc e po [pc]$ >> ] ]
+          <:expr< $mparser_match loc "Fstream" e po [pc]$ >> ] ]
   ;
   parser_case:
     [ [ "[:"; sp = stream_patt; ":]"; po = OPT ipatt; "->"; e = expr ->
@@ -231,7 +275,8 @@ EXTEND
   stream_patt_comp:
     [ [ "`"; p = patt; eo = OPT [ "when"; e = expr -> e ] -> SpTrm loc p eo
       | p = patt; "="; e = expr -> SpNtr loc p e
-      | p = patt -> SpStr loc p ] ]
+      | p = patt -> SpStr loc p
+      | "when"; e = expr  -> SpWhn loc e ] ]
   ;
   ipatt:
     [ [ i = LIDENT -> <:patt< $lid:i$ >> ] ]
