@@ -1,6 +1,6 @@
 (* camlp5r pa_extend.cmo q_MLast.cmo *)
-(* $Id: pa_r.ml 1269 2007-10-01 06:03:45Z deraugla $ *)
-(* Copyright (c) INRIA 2007 *)
+(* $Id: pa_r.ml 1812 2007-12-27 19:50:50Z deraugla $ *)
+(* Copyright (c) INRIA 2007-2008 *)
 
 open Pcaml;
 
@@ -343,7 +343,8 @@ EXTEND
     [ [ -> () ] ]
   ;
   sequence:
-    [ [ "let"; rf = V (FLAG "rec"); l = V (LIST1 let_binding SEP "and");
+    [ RIGHTA
+      [ "let"; rf = V (FLAG "rec"); l = V (LIST1 let_binding SEP "and");
         "in"; el = SELF ->
           [<:expr< let $_flag:rf$ $_list:l$ in $mksequence loc el$ >>]
       | e = expr; ";"; el = SELF -> [e :: el]
